from . import HermesTestCase
from .. import models, settings


class PostTestCase(HermesTestCase):
    def test_short(self):
        """A Post should return the truncated body if there is no summary"""
        expected = (
            u"<p>I've got to find a way to escape the horrible ravages of youth. "
            u"Suddenly, I'm going to the bathroom like clockwork, every three "
            u"hours. And those jerks at...</p>"
        )

        self.assertEqual(expected, self.post1.short)

    def test_short_summary_rendered(self):
        """Post.short should return the rendered summary"""
        expected = u"<h1>This is a summary</h1>"
        self.post1.summary = '#This is a summary'
        self.post1.save()

        self.assertEqual(expected, self.post1.short)

    def test_rendered(self):
        """A Post should be able to render its body into HTML"""
        self.post4.body = "##Markdown FTW!"
        self.post4.save()

        expected = "<h2>Markdown FTW!</h2>"

        self.assertEqual(expected, self.post4.rendered)

    def test_rendered_summary(self):
        """A Post should be able to render its summary into HTML"""
        self.post4.summary = "##Markdown FTW!"
        self.post4.save()

        expected = "<h2>Markdown FTW!</h2>"

        self.assertEqual(expected, self.post4.rendered_summary)

    def test_rendereded_no_renderer(self):
        """A Post should return its body if no renderer is defined"""
        renderer = settings.MARKUP_RENDERER
        settings.MARKUP_RENDERER = None

        expected = self.post1.body
        self.assertEqual(expected, self.post1.rendered)

        settings.MARKUP_RENDERER = renderer

    def test_hero_upload_to(self):
        """The Post Hero Upload To method should know the path to save the Hero image"""
        expected = "hermes/heroes/leela-her-own_hero.jpg"
        self.assertEqual(expected, models.post_hero_upload_to(self.post1, 'test.jpg'))

        expected = "hermes/heroes/raging-bender_hero.jpg"
        self.assertEqual(expected, models.post_hero_upload_to(self.post3, 'test.jpg'))

    def test_unicode(self):
        """The Post should have a unicode representation"""
        expected = u"A Tale of Two Santas"
        self.assertEqual(expected, self.post2.__unicode__())

        expected = u"The Series Has Landed"
        self.assertEqual(expected, self.post4.__unicode__())

    def test_get_absolute_url(self):
        """The Post should know its absolute URL"""
        expected = "/2010/06/10/leela-her-own/"
        self.assertEqual(expected, self.post1.get_absolute_url())

        expected = "/2013/09/10/series-has-landed/"
        self.assertEqual(expected, self.post4.get_absolute_url())

    def test_reading_time(self):
        """A Post should know how long it would take a human to read it."""
        expected = 20
        self.post1.body = "test " * 6000
        self.post1.save()
        self.assertEqual(expected, self.post1.reading_time)


class PostQuerySetTestCase(HermesTestCase):
    def test_reverse_creation_order(self):
        """The Post QuerySet should return Posts in reverse creation order"""
        expected = [self.post4, self.post3, self.post2, self.post1, ]
        self.assertEqual(expected, list(models.Post.objects.all()))

    def test_in_category(self):
        """The Post QuerySet should return Posts in a specific Category"""
        expected = [self.post3, self.post2, self.post1, ]
        self.assertEqual(expected, list(models.Post.objects.in_category('programming')))

    def test_in_category_children(self):
        """The Post QuerySet should return Posts in a specific Category and its Children"""
        self.post1.category = self.third_category
        self.post1.save()

        self.post2.category = self.second_category
        self.post2.save()

        expected = [self.post2, self.post1, ]
        self.assertEqual(expected, list(models.Post.objects.in_category('programming/python')))

    def test_recent(self):
        """The PostQuerySet recent method should return the most recent Posts up to 'limit'"""
        expected = [self.post4, self.post3, ]
        self.assertEqual(expected, list(models.Post.objects.recent(limit=2)))

    def test_recent_no_limit(self):
        """The PostQuerySet recent method should return all the Posts if no limit is set"""
        expected = [self.post4, self.post3, self.post2, self.post1, ]
        self.assertEqual(expected, list(models.Post.objects.recent(limit=None)))

    def test_random(self):
        """The PostQuerySet random method should return a random set of Posts"""
        unexpected = models.Post.objects.random(limit=3)
        self.assertNotEqual(unexpected, models.Post.objects.random(limit=3))
        self.assertTrue(
            all(
                isinstance(post, models.Post)
                for post in models.Post.objects.random(limit=3)
            )
        )

    def test_created_on_year(self):
        """The PostQuerySet should know which Posts were created in which year"""
        expected = [self.post1, ]
        self.assertEqual(expected, list(models.Post.objects.created_on(year=2010)))

        expected = [self.post2, ]
        self.assertEqual(expected, list(models.Post.objects.created_on(year=2011)))

    def test_created_on_month_year(self):
        """The PostQuerySet should know which Posts were created in which month/year"""
        expected = [self.post3, ]
        self.assertEqual(expected, list(models.Post.objects.created_on(year=2012, month=8)))

        expected = [self.post4, ]
        self.assertEqual(expected, list(models.Post.objects.created_on(year=2013, month=9)))

    def test_created_on_month_year_day(self):
        """The PostQuerySet should know which Posts were created in which day/month/year"""
        expected = [self.post1, ]
        self.assertEqual(
            expected, list(models.Post.objects.created_on(year=2010, month=6, day=10)))

        expected = [self.post3, ]
        self.assertEqual(
            expected, list(models.Post.objects.created_on(year=2012, month=8, day=10)))



from django import test
from django.core import urlresolvers
from django.contrib import auth


from .. import models


class HermesTestCase(test.TestCase):
    fixtures = ('hermes', )

    def setUp(self):
        self.root_category = models.Category.objects.get(id=1)
        self.second_category = models.Category.objects.get(id=2)
        self.third_category = models.Category.objects.get(id=3)
        self.another_category = models.Category.objects.get(id=4)

        self.post1 = models.Post.objects.get(id=1)
        self.post2 = models.Post.objects.get(id=2)
        self.post3 = models.Post.objects.get(id=3)
        self.post4 = models.Post.objects.get(id=4)

        self.user = auth.models.User.objects.get(id=1)

        self.client = test.Client()

    def url(self, url_name, *args, **kwargs):
        return urlresolvers.reverse(url_name, args=args, kwargs=kwargs)

    def get(self, url):
        return self.client.get(url)



import os
import operator
from functools import reduce
try:
    from itertools import ifilter as filter
except:
    pass  # Must be python 3

from django.contrib.auth.models import User
from django.db import models
from django.utils.text import Truncator, slugify
from django.utils.translation import ugettext as _

from . import settings


class TimestampedModel(models.Model):
    created_on = models.DateTimeField(_('created on'), auto_now_add=True)
    modified_on = models.DateTimeField(_('modified on'), auto_now=True)

    class Meta:
        abstract = True


class CategoryManager(models.Manager):
    def children_of(self, category, categories=None):
        if categories is None:
            categories = list(self.all())

        children = list(filter(lambda c: c.parent == category, categories))
        for child in children:
            children.extend(self.children_of(child, categories))

        return children


class Category(models.Model):
    title = models.CharField(_('title'), max_length=100)
    parent = models.ForeignKey('self', blank=True, null=True)
    slug = models.CharField(blank=True, default='', max_length=500, db_index=True)

    objects = CategoryManager()

    class Meta:
        verbose_name = u'category'
        verbose_name_plural = u'categories'

    def save(self, *args, **kwargs):
        self.slug = self._generate_slug()
        super(Category, self).save(*args, **kwargs)

    def __unicode__(self):
        return u" > ".join([category.title for category in self.hierarchy()])

    @models.permalink
    def get_absolute_url(self):
        return ('hermes_category_post_list', (), {
            'slug': self.slug,
        })

    def _generate_slug(self):
        return "/".join([slugify(category.title) for category in self.hierarchy()]).lower()

    @property
    def is_root(self):
        """ Returns True if this category has no parent. """
        return self.parent is None

    def parents(self):
        """ Returns a list of all the current category's parents."""
        parents = []

        if self.parent is None:
            return []

        category = self
        while category.parent is not None:
            parents.append(category.parent)
            category = category.parent

        return parents[::-1]

    def hierarchy(self):
        return self.parents() + [self]

    def root_parent(self, category=None):
        """ Returns the topmost parent of the current category. """
        return next(filter(lambda c: c.is_root, self.hierarchy()))


class PostQuerySet(models.query.QuerySet):
    def in_category(self, category_slug):
        category = Category.objects.get(slug=category_slug)
        children = Category.objects.children_of(category)

        return self.filter(category__in=[category] + children)

    def created_on(self, year=None, month=None, day=None):
        clauses = []

        if year:
            clauses.append(models.Q(created_on__year=year))

        if month:
            clauses.append(models.Q(created_on__month=month))

        if day:
            clauses.append(models.Q(created_on__day=day))

        return self.filter(reduce(operator.__and__, clauses))

    def recent(self, limit=None):
        queryset = self.all()
        if limit:
            queryset = queryset[:limit]

        return queryset

    def random(self, limit=None):
        queryset = self.recent().order_by('?')
        if limit:
            queryset = queryset[:limit]

        return queryset


class PostManager(models.Manager):
    def get_queryset(self):
        return PostQuerySet(Post)

    def __getattr__(self, attr, *args):
        try:
            return getattr(self.__class__, attr, *args)
        except AttributeError:
            return getattr(self.get_queryset(), attr, *args)


def post_hero_upload_to(instance, filename):
    extension = os.path.splitext(filename)[1][1:]

    return "hermes/heroes/{slug}_hero.{extension}".format(
        slug=instance.slug,
        extension=extension
    )


class Post(TimestampedModel):
    hero = models.ImageField(_('hero'), upload_to=post_hero_upload_to, blank=True, null=True)
    subject = models.CharField(_('subject'), max_length=100)
    slug = models.SlugField(_('slug'), max_length=100)
    summary = models.TextField(_('summary'), blank=True, null=True)
    body = models.TextField(_('body'))

    category = models.ForeignKey(Category)
    author = models.ForeignKey(User)

    objects = PostManager()

    class Meta:
        ordering = ('-created_on', )

    def __unicode__(self):
        return self.subject

    @models.permalink
    def get_absolute_url(self):
        return ('hermes_post_detail', (), {
            'year': self.created_on.year,
            'month': self.created_on.strftime('%m'),
            'day': self.created_on.strftime('%d'),
            'slug': self.slug,
        })

    @property
    def short(self):
        if self.summary:
            return self.rendered_summary
        else:
            return Truncator(self.rendered).words(settings.HERMES_SHORT_TRUNCATE_WORDS, html=True)

    @property
    def rendered_summary(self):
        return self._rendered_attribute('summary')

    @property
    def rendered(self):
        return self._rendered_attribute('body')

    def _rendered_attribute(self, attr_name):
        attr_value = getattr(self, attr_name)
        if settings.MARKUP_RENDERER:
            return settings.MARKUP_RENDERER(attr_value)
        else:
            return attr_value

    @property
    def reading_time(self):
        time = (self.summary.count(' ') + self.body.count(' ')) / 300
        if time == 0:
            time = 1
        return time



from django.views.generic import ListView, DetailView

from .models import Post


class PostListView(ListView):
    context_object_name = 'posts'
    model = Post
    template_name = 'hermes/post_list.html'


class CategoryPostListView(PostListView):
    def get_queryset(self):
        category_slug = self.kwargs.get('slug', '')
        return self.model.objects.in_category(category_slug)


class ArchivePostListView(PostListView):
    def get_queryset(self):
        year = self.kwargs.get('year', None)
        month = self.kwargs.get('month', None)
        day = self.kwargs.get('day', None)

        return self.model.objects.created_on(year=year, month=month, day=day)


class PostDetail(DetailView):
    context_object_name = 'post'
    model = Post
    template_name = "hermes/post_detail.html"



from . import HermesTestCase
from .. import models


class CategoryTestCase(HermesTestCase):
    def test_is_root(self):
        """A Category should know if it is the root category"""
        expected = True
        self.assertEqual(expected, self.root_category.is_root)

        expected = False
        self.assertEqual(expected, self.second_category.is_root)

        expected = False
        self.assertEqual(expected, self.third_category.is_root)

    def test_hierarchy(self):
        """A Category should know it's hierarchy"""
        expected = [self.root_category, ]
        self.assertEqual(expected, self.root_category.hierarchy())

        expected = [self.root_category, self.second_category, ]
        self.assertEqual(expected, self.second_category.hierarchy())

        expected = [self.root_category, self.second_category, self.third_category, ]
        self.assertEqual(expected, self.third_category.hierarchy())

    def test_parents(self):
        """A Category should know its parents"""
        expected = []
        self.assertEqual(expected, self.root_category.parents())

        expected = [self.root_category, ]
        self.assertEqual(expected, self.second_category.parents())

        expected = [self.root_category, self.second_category, ]
        self.assertEqual(expected, self.third_category.parents())

    def test_root_parent(self):
        """A Category should know its top-most parent"""
        expected = self.root_category
        self.assertEqual(expected, self.root_category.root_parent())
        self.assertEqual(expected, self.second_category.root_parent())
        self.assertEqual(expected, self.third_category.root_parent())

        expected = self.another_category
        self.assertEqual(expected, self.another_category.root_parent())

    def test_generate_slug(self):
        """A Category should know how to generate its slug"""
        expected = u'programming'
        self.assertEqual(expected, self.root_category._generate_slug())

        expected = u'programming/python'
        self.assertEqual(expected, self.second_category._generate_slug())

        expected = u'programming/python/django'
        self.assertEqual(expected, self.third_category._generate_slug())

    def test_unicode(self):
        """A Category should have a unicode representation"""
        expected = u'Programming'
        self.assertEqual(expected, self.root_category.__unicode__())

        expected = u'Programming > Python'
        self.assertEqual(expected, self.second_category.__unicode__())

        expected = u'Programming > Python > Django'
        self.assertEqual(expected, self.third_category.__unicode__())

    def test_get_absolute_url(self):
        """A Category should know its absolute URL"""
        expected = u'/categories/programming/'
        self.assertEqual(expected, self.root_category.get_absolute_url())

        expected = u'/categories/programming/python/'
        self.assertEqual(expected, self.second_category.get_absolute_url())

        expected = u'/categories/food/'
        self.assertEqual(expected, self.another_category.get_absolute_url())

    def test_save(self):
        """A Category should update its slug on save"""
        self.third_category.slug = u"Banana Slug"
        self.third_category.save()

        expected = u'programming/python/django'
        self.assertEqual(expected, self.third_category.slug)


class CategoryManagerTestCase(HermesTestCase):
    def test_children_of(self):
        """The Category Manager should know the children of a Category"""
        expected = [self.second_category, self.third_category, ]
        self.assertEqual(expected, models.Category.objects.children_of(self.root_category))

        expected = [self.third_category, ]
        self.assertEqual(expected, models.Category.objects.children_of(self.second_category))

    def test_children_of_leaf(self):
        """The Category Manager should know that a leaf Category has no children"""
        expected = []
        self.assertEqual(expected, models.Category.objects.children_of(self.third_category))



from setuptools import setup, find_packages

setup(
    author='Chris Pickett',
    author_email='chris.pickett@gmail.com',
    description='A light-weight blogging app for Django.',
    license='MIT',
    long_description=open('README.rst').read(),
    name='django-hermes',
    packages=find_packages(),
    url='https://github.com/bunchesofdonald/django-hermes',
    version='1.3.0',
)
